==> wurld-0.3.tar.gz: wurld-0.3/src/Moveable.h <==
#ifndef MOVEABLE_H
#define MOVEABLE_H

#include <spug/RCPtr.h>
#include "Model.h"
#include "GameObject.h"
#include "RenderableObject.h"
#include "SimpleCollidable.h"
#include "MoveableAlgo.h"
#include <spug/STLIteratorImpl.h>
#include "CollisionHandler.h"

SPUG_RCPTR(CollisionAlgo);
class GameEngine;

/**
 * A renderable object that has a position and an orientation.  There is also
 * an associated model which can be modified.
 *
 * A null value can be used for the model - this effectively makes the object
 * invisible.
 */
class Moveable : public RenderableObject, public GameObject {

   private:
      ModelPtr model;
      Vector position, velocity;
      RotMatrix rotation;

      typedef std::vector<SimpleCollidablePtr> CollidableVec;
      CollidableVec collidables;
      CollisionAlgoPtr collisionAlgo;

      // target of the next collision
      CollidablePtr collisionTarget;

      // collision handler to be called on the next collision
      CollisionHandler *collisionHandler;

      typedef std::vector<MoveableAlgoPtr> AlgoVec;
      AlgoVec algos;

      // the maximum distance that the object is allowed to travel before it
      // collides with another object
      Float maxDistance;

      bool trace;

   public:
      typedef spug::Iterator<SimpleCollidablePtr> CollidableIterator;

      Moveable();

      Moveable(ModelPtr model, const Vector &position, 
	       const RotMatrix &rotation
	       );

      ~Moveable();

      virtual void render(Scene &scene);

      /** 
       * Sets the position vector.
       * "position" is the absolute position of the object 
       */
      void setPosition(const Vector &newPosition) {
	 position = newPosition;
      }

      /** 
       * Sets the position vector.
       * "position" is the absolute position of the object 
       */
      const Vector &getPosition() const {
	 return position;
      }

      /**
       * Sets the velocity vector.
       */
      void setVelocity(const Vector &newVelocity) {
         velocity = newVelocity;
         maxDistance = infinity;
      }

      /**
       * Returns the velocity vector.
       */
      const Vector &getVelocity() const {
         return velocity;
      }

      /**
       * Sets the rotation of the object.
       */
      void setRotation(const RotMatrix &newRotation) {
	 rotation = newRotation;
      }

      /**
       * Returns the rotation of the object.
       */
      RotMatrix getRotation() const {
         return rotation;
      }

      /**
       * Sets the associated model.
       */
      void setModel(const ModelPtr &newModel) {
	 model = newModel;
      }

      /**
       * Returns an iterator over the sequence of "collidables."
       */
      CollidableIterator getCollidables() const {
         typedef 
               spug::STLIteratorImpl<SimpleCollidablePtr, 
                                     std::vector<SimpleCollidablePtr>,
                                     std::vector<SimpleCollidablePtr>::const_iterator
                                     >
               IterImpl;
         return IterImpl::alloc(collidables.begin(), collidables.end());
      }      

      /** 
       * Get/Set the maximum distance the object can travel before colliding
       * with something else.
       */
      /** @{ */
      void setMaxDistance(Float dist) {
	 if (trace) std::cout << "set max distance to " << dist << std::endl;
         maxDistance = dist;
      }

      Float getMaxDistance() const {
	 return maxDistance;
      }
      /** @} */

      /**
       * Sets the target of the next collision.
       */
      void setCollisionTarget(const CollidablePtr &newCollisionTarget) {
         collisionTarget = newCollisionTarget;
      }

      /**
       * Adds a new algorithm.  Algorithms are added to the end of the chain -
       * algorithms added first are executed first.
       */
      void addAlgo(const MoveableAlgoPtr &algo);

      /**
       * Removes an algorithm from the chain.  Throws
       * spug::DoesNotExistException if the algorithm is not currently
       * registered.
       */
      void removeAlgo(const MoveableAlgoPtr &algo);

      /**
       * Adds a new collidable to the object.
       */
      void addCollidable(const SimpleCollidablePtr &collidable);

      /**
       * Sets the collision algorithm.
       */
      void setCollisionAlgo(const CollisionAlgoPtr &algo);

      /**
       * checks for a collision with the other collidable.
       */
      bool checkForCollision(const CollidablePtr &other, Float &time);

      /**
       * Informs the moveable of a collision - this may not necessarily be the
       * first collision that the moveable will encounter.
       */
      void informOfCollision(const CollidablePtr &other, Float &distance);

      /**
       * turns tracing on and off
       */
      void enableTrace(bool newTrace) {
	 trace = newTrace;
      }

      /** @{ */
      /** 
       * Accessors for the collision handler (this is the function that gets
       * called when a collision occurs).
       */
      void setCollisionHandler(CollisionHandler *newCollisionHandler) {
         collisionHandler = newCollisionHandler;
      }

      CollisionHandler *getCollisionHandler() const {
         return collisionHandler;
      }
      /** @} */

      // Implements GameObject
      virtual void processRenderFrame(GameEngine &engine, Float elapsed);

      virtual void processEngineFrame(GameEngine &engine);
};

SPUG_RCPTR(Moveable);

#endif

==> wurld-0.4.tar.gz: wurld-0.4/src/Moveable.h <==
#ifndef MOVEABLE_H
#define MOVEABLE_H

#include <spug/RCPtr.h>
#include "Model.h"
#include "GameObject.h"
#include "RenderableObject.h"
#include "SimpleCollidable.h"
#include "MoveableAlgo.h"
#include <spug/STLIteratorImpl.h>
#include "CollisionHandler.h"

SPUG_RCPTR(CollisionAlgo);
class GameEngine;

/**
 * A renderable object that has a position and an orientation.  There is also
 * an associated model which can be modified.
 *
 * A null value can be used for the model - this effectively makes the object
 * invisible.
 */
class Moveable : public RenderableObject, public GameObject {

   private:
      ModelPtr model;
      Vector position, velocity;
      RotMatrix rotation;

      typedef std::vector<SimpleCollidablePtr> CollidableVec;
      CollidableVec collidables;
      CollisionAlgoPtr collisionAlgo;

      // target of the next collision
      CollidablePtr collisionTarget;

      // collision handler to be called on the next collision
      CollisionHandler *collisionHandler;

      typedef std::vector<MoveableAlgoPtr> AlgoVec;
      AlgoVec algos;

      // the maximum distance that the object is allowed to travel before it
      // collides with another object
      Float maxDistance;

      bool trace;

   public:
      typedef spug::Iterator<SimpleCollidablePtr> CollidableIterator;

      Moveable();

      Moveable(ModelPtr model, const Vector &position, 
	       const RotMatrix &rotation
	       );

      ~Moveable();

      virtual void render(Scene &scene);

      /** 
       * Sets the position vector.
       * "position" is the absolute position of the object 
       */
      void setPosition(const Vector &newPosition) {
	 position = newPosition;
      }

      /** 
       * Sets the position vector.
       * "position" is the absolute position of the object 
       */
      const Vector &getPosition() const {
	 return position;
      }

      /**
       * Sets the velocity vector.
       */
      void setVelocity(const Vector &newVelocity) {
         velocity = newVelocity;
         maxDistance = infinity;
      }

      /**
       * Returns the velocity vector.
       */
      const Vector &getVelocity() const {
         return velocity;
      }

      /**
       * Sets the rotation of the object.
       */
      void setRotation(const RotMatrix &newRotation) {
	 rotation = newRotation;
      }

      /**
       * Returns the rotation of the object.
       */
      RotMatrix getRotation() const {
         return rotation;
      }

      /**
       * Sets the associated model.
       */
      void setModel(const ModelPtr &newModel) {
	 model = newModel;
      }

      /**
       * Returns an iterator over the sequence of "collidables."
       */
      CollidableIterator getCollidables() const {
         typedef 
               spug::STLIteratorImpl<SimpleCollidablePtr, 
                                     std::vector<SimpleCollidablePtr>,
                                     std::vector<SimpleCollidablePtr>::const_iterator
                                     >
               IterImpl;
         return IterImpl::alloc(collidables.begin(), collidables.end());
      }      

      /** 
       * Get/Set the maximum distance the object can travel before colliding
       * with something else.
       */
      /** @{ */
      void setMaxDistance(Float dist) {
	 if (trace) std::cout << "set max distance to " << dist << std::endl;
         maxDistance = dist;
      }

      Float getMaxDistance() const {
	 return maxDistance;
      }
      /** @} */

      /**
       * Sets the target of the next collision.
       */
      void setCollisionTarget(const CollidablePtr &newCollisionTarget) {
         collisionTarget = newCollisionTarget;
      }

      /**
       * Adds a new algorithm.  Algorithms are added to the end of the chain -
       * algorithms added first are executed first.
       */
      void addAlgo(const MoveableAlgoPtr &algo);

      /**
       * Removes an algorithm from the chain.  Throws
       * spug::DoesNotExistException if the algorithm is not currently
       * registered.
       */
      void removeAlgo(const MoveableAlgoPtr &algo);

      /**
       * Adds a new collidable to the object.
       */
      void addCollidable(const SimpleCollidablePtr &collidable);

      /**
       * Sets the collision algorithm.
       */
      void setCollisionAlgo(const CollisionAlgoPtr &algo);

      /**
       * checks for a collision with the other collidable.
       */
      bool checkForCollision(const CollidablePtr &other, Float &distance);

      /**
       * Informs the moveable of a collision - this may not necessarily be the
       * first collision that the moveable will encounter.
       */
      void informOfCollision(const CollidablePtr &other, Float &distance);

      /**
       * turns tracing on and off
       */
      void enableTrace(bool newTrace) {
	 trace = newTrace;
      }

      /** @{ */
      /** 
       * Accessors for the collision handler (this is the function that gets
       * called when a collision occurs).
       */
      void setCollisionHandler(CollisionHandler *newCollisionHandler) {
         collisionHandler = newCollisionHandler;
      }

      CollisionHandler *getCollisionHandler() const {
         return collisionHandler;
      }
      /** @} */

      // Implements GameObject
      virtual void processRenderFrame(GameEngine &engine, Float elapsed);

      virtual void processEngineFrame(GameEngine &engine);
};

SPUG_RCPTR(Moveable);

#endif
